package org.netkernel.ext.system.init;

import java.net.URI;
import java.util.Iterator;

import org.netkernel.container.config.IConfiguration;
import org.netkernel.container.config.IConfigurationListener;
import org.netkernel.layer0.boot.BootUtils;
import org.netkernel.layer0.boot.IModuleManagerSyncListener;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.module.standard.endpoint.StandardTransportImpl;

public class InitEndpoint extends StandardTransportImpl implements IConfigurationListener
{	private boolean mBooted;

	
	private static final int LOAD_ALL_LEVELS=-1;
	@Override
	protected void postCommission(INKFRequestContext context) throws Exception
	{	if(!mBooted)
		{	ModuleManager mm=ModuleManager.getSingleton();
	 		mm.registerSyncListener(new InitModuleManagerSyncListener());
	 		context.getKernelContext().getKernel().addConfigurationListener(this);
		}
	}
	
	private void initBoot() throws Exception
	{	mBooted=true;
		getTransportContext().logFormatted(INKFRequestContext.LEVEL_INFO,"MSG_INIT_STARTED", null);
		IConfiguration config=getTransportContext().getKernelContext().getKernel().getConfiguration();
		
		int limitlevel=LOAD_ALL_LEVELS;
		try
		{	String limitlevelstr=config.getString("netkernel.init.runlevel");
			limitlevel=Integer.parseInt(limitlevelstr);
		}
		catch(Exception e){ /*Do nothing */}
		
		if(BootUtils.bootedFromJar(config))
		{	System.out.println("Init not implemented from Jar boot yet")	;
		}
		else
		{	IHDSNode modHDS=null;
			URI moduleListURI=BootUtils.getInitModules(config);
			if(moduleListURI!=null)
			{	String scheme=moduleListURI.getScheme();
				if(scheme.startsWith("http"))
				{	//Add wormhole access to http: remote module list loading
					System.out.println("Init remote module list not supported yet.");
				}
				else if(scheme.equals("file"))
				{	URI base=URI.create(BootUtils.getInstallPath(config));
					modHDS=getTransportContext().source(moduleListURI.toString(), IHDSNode.class);
					if(modHDS!=null)
					{	IHDSNodeList l=modHDS.getNodes("modules/module");
						Iterator< IHDSNode> it=l.iterator();
						ModuleManager mm=ModuleManager.getSingleton();
						while(it.hasNext())
						{	boolean shouldadd=true;
							IHDSNode n=it.next();
							URI u=base.resolve((String)n.getValue());
							String levels=(String)n.getFirstValue("@runlevel");
							int level=ModuleManager.DEFAULT_RUN_LEVEL;
							if(levels!=null)
							{	try
								{	level=Integer.parseInt(levels);
								}
								catch(Exception e)
								{	getTransportContext().logRaw(INKFRequestContext.LEVEL_WARNING, "Invalid runlevel for "+u.toString());
									level=ModuleManager.DEFAULT_RUN_LEVEL;
								}
								if(level<1)
								{	getTransportContext().logFormatted(INKFRequestContext.LEVEL_WARNING,"WARN_INIT_LEVEL_TOO_LOW",new Object[]{u.toString(), ModuleManager.DEFAULT_RUN_LEVEL});
									level=ModuleManager.DEFAULT_RUN_LEVEL;
								}
							}
							if(shouldadd)
							{	mm.addModule(u, level);
							}
							
						}
						Boolean incremental=config.getBoolean("netkernel.init.incremental", false);
						if(incremental)
						{	for(int steplevel=2; steplevel<limitlevel+1; steplevel++)
							{	mm.setRunLevel(steplevel);
							}
						}
						else
						{	mm.setRunLevel(limitlevel);
						}
					}
				}
				else
				{	System.err.println("Init does not support scheme for "+moduleListURI);
				}
			}
			else
			{	System.err.println("Init cannot load modules netkernel.init.modules property not found");
			}
		}
		getTransportContext().logFormatted(INKFRequestContext.LEVEL_INFO,"MSG_INIT_COMPLETED",new Object[]{limitlevel});
	}
	
	@Override
	protected void preDecommission(INKFRequestContext context) throws Exception
	{
 		context.getKernelContext().getKernel().removeConfigurationListener(this);
	}
	
	class InitModuleManagerSyncListener implements IModuleManagerSyncListener
	{
		public void syncComplete()
		{	ModuleManager.getSingleton().registerSyncListener(null);
			try
			{	initBoot();
			}
			catch(Exception e)
			{	e.printStackTrace();		
			}
		}
	}

	public void configurationChanged(IConfiguration aConfiguration)
	{	
		if (mBooted)
		{
			int runlevel=aConfiguration.getInt("netkernel.init.runlevel", 5);
			ModuleManager.getSingleton().setRunLevel(runlevel);
		}
	}		
}
